@@ -393,7 +393,8 @@ else:
             whatTgyroDict[r'$\omega_0$'] = {'normalization': [1e3, 1e3], 'unit': '$krad/s$', 'name': ['omega0', 'w0']}
 
             for k, whatTgyro in enumerate(whatTgyroDict.keys()):
-                pyplot.gcf().add_subplot(2, 3, k + 1)
+                pyplot.gcf()
+                pyplot.subplot(2, 3, k + 1)
                 if whatTgyro == 'convergence':
                     pyplot.semilogy(self['convergence'], color='k')
                     pyplot.xlim([0, len(self['convergence']) - 1])
