@@ -52,7 +52,6 @@ __all__ = [
     'OMFITmdsCache',
     'interpret_signal',
     'available_efits_from_mds',
-    'check_efit_tree_validity',
 ]
 
 D3D_mds_server_trename_selector = {'': 'atlas', 'lbo': 'orion'}
@@ -138,25 +137,28 @@ my_ip_address = [get_ip()]
 def runFunctionCatchExceptions(func, *args, **kwargs):
     try:
         result = func(*args, **kwargs)
-    except Exception as  message:
+    except Exception as message:
         return "exception", message
 
     return "RESULT", result
 
 
 def runFunctionWithTimeout(func, *args, **kwargs):
-    timeout = 10 #s
+    timeout = 10  # s
     import threading, time
+
     class InterruptableThread(threading.Thread):
         def __init__(self):
             threading.Thread.__init__(self)
             self.result = None
+
         def run(self):
             self.result = runFunctionCatchExceptions(func, *args, **kwargs)
+
     it = InterruptableThread()
     it.start()
     it.join(timeout)
- 
+
     if it.is_alive():
         return False, None
 
@@ -164,17 +166,19 @@ def runFunctionWithTimeout(func, *args, **kwargs):
         raise it.result[1]
 
     return True, it.result[1]
-    
-#raise error of the tree cannot be opened in 5 seconds
+
+
+# raise error of the tree cannot be opened in 5 seconds
 def safeOpenTree(conn, tree, shot):
     if conn is None:
         raise OMFITexception('MDS+ connection was unsuccesful')
-        
-    finished, _ = runFunctionWithTimeout(conn.openTree,  tree, shot)
+
+    finished, _ = runFunctionWithTimeout(conn.openTree, tree, shot)
     if not finished:
         raise OMFITexception('Opening of MDS+ tree was unsuccesful')
-        
-#raise error of the tree cannot be connected in 5 seconds
+
+
+# raise error of the tree cannot be connected in 5 seconds
 def safeMDSConnect(server):
     finished, conn = runFunctionWithTimeout(MDSplus.Connection, server)
 
@@ -183,7 +187,7 @@ def safeMDSConnect(server):
 
     return conn
 
-    
+
 def ip_check_reset():
     """reset connections when IP changes, this is necessary to avoid long timeout hardcoded in C part of MDS+"""
     current_ip = get_ip()
@@ -559,7 +563,7 @@ class OMFITmdsConnectionBaseClass(SortedDict):
             )
         self.timeout = timeout
         self.last_open_tree = {}
-        
+
     def mdsvalue(self, server, treename, shot, what, timeout=None):
         warnings.filterwarnings('ignore', message=r"tostring\(\) is deprecated\. Use tobytes\(\) instead\.", category=DeprecationWarning)
 
@@ -643,11 +647,11 @@ class OMFITmdsConnectionBaseClass(SortedDict):
                             )
 
                         elif treename is not None:
-                            #Open new tree only if it not already opened
-                            if self.last_open_tree.get(mds_connection_cache_ID,'') != (treename,int(shot)) :
+                            # Open new tree only if it not already opened
+                            if self.last_open_tree.get(mds_connection_cache_ID, '') != (treename, int(shot)):
                                 safeOpenTree(self[mds_connection_cache_ID], treename, int(shot))
                                 self.last_open_tree[mds_connection_cache_ID] = treename, int(shot)
-                                
+
                             try:
                                 queryResult = mdsplus_data(self[mds_connection_cache_ID].get(pre + signal + post, timeout=timeout))
                                 printd(f'* Retrieving data {treename} for shot {shot}', topic='MDS')
@@ -794,7 +798,7 @@ class OMFITmdsConnectionBaseClass(SortedDict):
         mds_connection_cache_ID = mds_connection_cache_ID_generator('write', server, treename, shot)
         if mds_connection_cache_ID not in self:
             self[mds_connection_cache_ID] = safeMDSConnect(server)
-            safeOpenTree(self[mds_connection_cache_ID],treename, shot)
+            safeOpenTree(self[mds_connection_cache_ID], treename, shot)
         conn = self[mds_connection_cache_ID]
 
         # generate TDI expression for writing data to MDS+
@@ -1649,6 +1653,10 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
                 return dict(data_failed_early=True)
             return False
 
+        if np.size(data) == 1:
+            # do not check the timebase for scalers
+            check_dim_of = -1
+
         # If treename is None, the MDS wrapper can check DIII-D's PTDATA system. A failed call here returns data = [0].
         if self.treename is None:
             bad_ptdata = len(np.atleast_1d(data)) == 1 and np.atleast_1d(data)[0] == 0
@@ -1718,33 +1726,7 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
     # End of class OMFITmdsValue
 
 
-def check_efit_tree_validity(device, shot, treename):
-    """
-    Checks whether an EFIT tree is populated / appears to have valid results
-
-    :param device: str
-        Device name
-
-    :param shot: int
-        Shot number
-
-    :param treename: str
-        Name of the MDSplus tree for the specific EFIT result in question, like 'EFIT01'
-
-    :return: bool
-        True if the tree might be okay, False if the tree is not okay
-    """
-    try:
-        printd(f'check {device}#{shot} {treename}', topic='available_EFITs')
-        mds = OMFITmdsValue(device, shot=shot, treename=treename, TDI=rf'GETNCI("\\{treename}::TOP.RESULTS.GEQDSK.GTIME", "length")')
-        if mds.data() is None:
-            return False
-        return mds.data()[0] > 0
-    except MDSplus.mdsExceptions.TreeFILE_NOT_FOUND:
-        return False
-
-
-def available_efits_from_mds(scratch_area, device, shot, default_snap_list=None, format='{tree}', no_empty=True, check_contents=False):
+def available_efits_from_mds(scratch_area, device, shot, list_empty_efits=False, default_snap_list=None, format='{tree}'):
     """
     Attempts to lookup a list of available EFITs from MDSplus
 
@@ -1757,12 +1739,7 @@ def available_efits_from_mds(scratch_area, device, shot, default_snap_list=None,
 
     If the device's MDSplus tree is not arranged like this, it will fail and return [].
 
-    The number of MDSplus calls made by this function is determined by:
-        no_empty = False, check_contents = False: 1 call
-        no_empty =  True, check_contents = False: 2 calls
-        no_empty = False, check_contents =  True: 1 + N calls
-        no_empty =  True, check_contents =  True: 2 + N calls
-    Where N is the number of EFIT results found.
+    Requires a single MDSplus call
 
     :param scratch_area: dict
         Scratch area for storing results to reduce repeat calls. Mainly included to match
@@ -1773,6 +1750,8 @@ def available_efits_from_mds(scratch_area, device, shot, default_snap_list=None,
 
     :param shot: int
         Shot number
+    :param list_empty_efits: bool
+        List all EFITs includeing these without any data
 
     :param default_snap_list: dict [optional]
         Default set of EFIT treenames. Newly discovered ones will be added to the list.
@@ -1781,15 +1760,6 @@ def available_efits_from_mds(scratch_area, device, shot, default_snap_list=None,
         Instructions for formatting data to make the EFIT tag name.
         Provided for compatibility with available_efits_from_rdb() because the only option is '{tree}'.
 
-    :param no_empty: bool
-        Use a second TDI command to check the number of nodes in each EFIT and
-        filter out EFIT trees that are empty (0 nodes within them).
-        Adds one MDSplus call.
-
-    :param check_contents: bool
-        Do a deeper check to make sure GEQDSK data are populated with at least one time.
-        Adds one MDSplus call per EFIT that was found so far.
-
     :return: (dict, str)
         Dictionary keys will be descriptions of the EFITs
             Dictionary values will be the formatted identifiers.
@@ -1809,19 +1779,19 @@ def available_efits_from_mds(scratch_area, device, shot, default_snap_list=None,
     efit_group_parent_treename = {  # None means that EFITs for this device are not organized under a single parent
         'DIII-D': None,
         'EAST': None,
-        'KSTAR': 'EFIT',
-        'NSTX': 'EFIT',
+        'KSTAR': ['EFIT'],
+        'NSTX': ['LRDFIT', 'EFIT'],
     }
-    efit_group_parent_treename['NSTX-U'] = efit_group_parent_treename['NSTX']
-    treename = efit_group_parent_treename.get(tokamak(device), None)
+    efit_group_parent_treename['NSTXU'] = efit_group_parent_treename['NSTX']
+    treenames = efit_group_parent_treename.get(tokamak(device), None)
 
     # Complain about problems
-    if treename is None:
+    if treenames is None:
         printd(f"  No rule for looking up all EFITs from MDSplus for {device}.", topic='available_EFITs')
         scratch_area['available_efits_from_mds_success'] = False
         return snap_list, help_info
     else:
-        printd(f'  EFITs for {device} should be stored in treename = {treename}.', topic='available_EFITs')
+        printd(f'  EFITs for {device} should be stored in treename = {treenames}.', topic='available_EFITs')
 
     if format != '{tree}':
         raise NotImplementedError(
@@ -1829,34 +1799,33 @@ def available_efits_from_mds(scratch_area, device, shot, default_snap_list=None,
             'Please use format="{tree}" intead.'
         )
 
-    # Try to gather information
-    mds = OMFITmdsValue(device, shot=shot, treename=treename, TDI=rf'GETNCI("\\{treename}::TOP.*", "MINPATH")')
-    if mds.check():
-        efits = [d.split('.')[-1] for d in mds.data()]
+    efits = []
+    # in case that there are multiple trees with EFIT subtrees
+    for treename in treenames:
+        # Try to gather valid EFITs
+        TDI = r'_y = getnci("\\' + treename + '::TOP.*.RESULTS","minpath");'
+        if not list_empty_efits:
+            TDI += r'_s = getnci("\\' + treename + '::TOP.*.RESULTS.GEQDSK:GTIME","length") > 0;'
+            TDI += r'PACK(_y,_s)'
+
+        mds = OMFITmdsValue(device, shot=shot, treename=treename, TDI=TDI)
+        if mds.check():
+            efits += [d.split('.')[1] for d in mds.data()]
+
+    if len(efits):
         help_info = f'Found list of EFITs in MDSplus for {device}#{shot}.'
-        # Options to filter the list
-        if no_empty:
-            tdi2 = rf'GETNCI("\\{treename}::TOP.*", "number_of_members")'
-            mds2 = OMFITmdsValue(device, shot=shot, treename=treename, TDI=tdi2)
-            validity = [d > 0 for d in mds2.data()]
-            if len(validity) == len(efits):
-                efits = list(array(efits)[array(validity)])
-            help_info += ' Removed EFITs that were totally empty.'
-        if check_contents:
-            validity2 = [check_efit_tree_validity(device, shot, efit) for efit in efits]
-            efits = list(array(efits)[array(validity2)])
-            help_info += ' Removed EFITs that had no GEQDSK time slices.'
         printd(f'  Found EFITs in MDSplus for {device}: {efits}', topic='available_EFITs')
         # Format the results
         for tree in efits:
             snap_list[f"[{tree}]"] = format.format(tree=tree)
         scratch_area['available_efits_from_mds_success'] = True
-        return snap_list, help_info
+
     else:
         printd(f'  Failed to lookup list of EFITs in MDSplus for {device}#{shot}.', topic='available_EFITs')
         scratch_area['available_efits_from_mds_success'] = False
         help_info = f'Failed to locate any EFITs in MDSplus for {device}#{shot}'
-        return snap_list, help_info
+
+    return snap_list, help_info
 
 
 # ==================
